import sys

from .version import VERSION


class LoggingConfiguration(object):
    LOGGER_NAME = "rook"
    FILE_NAME = "rookout/python-rook.log"
    LOG_TO_STDERR = False
    LOG_LEVEL = "INFO"
    PROPAGATE_LOGS = False
    LOG_TO_REMOTE = False
    DEBUG = False


class VersionConfiguration(object):
    VERSION = VERSION
    COMMIT = "CommitGoesHere"


class ProtobufConfiguration(object):
    NAMESPACE_SERIALIZER_DUMPING = r""


class ControllerAddress(object):
    HOST = 'wss://control.rookout.com'
    PORT = 443


class AgentComConfiguration(object):
    COMMAND_THREAD_NAME = "rookout_agent_com"
    MAX_MESSAGE_LENGTH = 100 * 1024 * 1024
    BACK_OFF = 0.2
    MAX_SLEEP = 60
    TIMEOUT = 3
    REQUEST_TIMEOUT_SECS = 30
    PING_TIMEOUT = 30
    PING_INTERVAL = 10
    RESET_BACKOFF_TIMEOUT = 3*60.0
    FLUSH_TIMEOUT = 3
    MAX_QUEUED_MESSAGES = 500


class OutputWsConfiguration(object):
    MAX_STATUS_UPDATES = 200
    MAX_LOG_ITEMS = 200
    MAX_AUG_MESSAGES = 100
    BUCKET_REFRESH_RATE = 10


class InstrumentationConfig(object):
    ENGINE = "auto"
    MIN_TIME_BETWEEN_HITS_MS = 100
    MAX_AUG_TIME = 400


class ImportServiceConfig(object):
    USE_IMPORT_HOOK = True
    SYS_MODULES_QUERY_INTERVAL = 0.25
    THREAD_NAME = "rookout_import_service_thread"


class HttpServerServiceConfig(object):
    SERVICES_NAMES = ""


class GitConfig(object):
    GIT_COMMIT = None
    GIT_ORIGIN = None



import six

from .namespace import Namespace

from .python_object_namespace import PythonObjectNamespace

from rook.exceptions import RookAttributeNotFound


class ContainerNamespace(dict, Namespace):
    def __init__(self, dictionary=None):
        Namespace.__init__(self, self.METHODS)
        dict.__init__(self, dictionary or {})

    def read_attribute(self, name):
        try:
            return self[name]
        except KeyError:
            raise RookAttributeNotFound(name)

    def write_attribute(self, name, value):
        self[name] = value

    def size(self, args):
        return PythonObjectNamespace(len(self))

    def __getattr__(self, item):
        if item == u'dictionary':
            return self
        elif item in self:
            return self[item]
        else:
            return dict.__getattribute__(self, item)

    METHODS = (size,)



"""This module is in charge of managing all the trigger services.

Trigger services are the components responsible managing the execution of Augs.

Each trigger service must export the following methods:
- __init__(self, output) - Construct the object, receives a point to the Output object.
- remove_aug(self, aug_id) - Remove an Aug by it's id.
- clear_augs(self) - Remove all Augs.

Services are loaded from the services packages.
"""
from six import itervalues

from .services import BdbLocationService, ImportService, LoggingLocationService, HttpServerService

from .exceptions import RookServiceMissing

from .logger import logger


class TriggerServices(object):
    """This class implements all the module's functionality."""
    def __init__(self):
        """Load and initialize all the trigger services."""
        self._services = dict()

    def get_service(self, name):
        """Return the trigger service by it's name."""
        try:
            return self._services[name]
        except KeyError:
            raise RookServiceMissing(name)

    def remove_aug(self, aug_id):
        """Remove an Aug from all services by it's id."""
        for service in itervalues(self._services):
            try:
                service.remove_aug(aug_id)
            except:
                logger.exception("Error when removing aug")

    def clear_augs(self):
        """Remove all Augs from all services."""

        for service in itervalues(self._services):
            service.clear_augs()

    def start(self):
        self._load_services()

    def close(self):
        for service in itervalues(self._services):
            service.close()

        self._services = {}

    def pre_fork(self):
        for service in itervalues(self._services):
            service.pre_fork()

    def post_fork(self):
        for service in itervalues(self._services):
            service.post_fork()

    def _load_services(self):
        try:
            self._services[BdbLocationService.NAME] = BdbLocationService()
            self._services[ImportService.NAME] = ImportService(self._services[BdbLocationService.NAME])
        except:
            logger.exception("Error starting location based services")

        try:
            self._services[LoggingLocationService.NAME] = LoggingLocationService()
        except:
            logger.exception("Error starting log handler services")

        try:
            self._services[HttpServerService.NAME] = HttpServerService()
        except:
            logger.exception("Error starting http server services")



from rook.logger import logger

from rook.processor.paths.arithmetic_path import ArithmeticPath
from rook.processor.namespaces.python_object_namespace import PythonObjectNamespace

_rdb = None
_kwargs = {}


def _get_rdb(**kwargs):
    global _rdb
    global _kwargs

    from . import rdb

    if kwargs and _kwargs != kwargs:
        if _rdb:
            _rdb._close_session()
            _rdb = None
            _kwargs = {}

        _rdb = rdb.Rdb(**kwargs)
        _kwargs = kwargs
    else:
        if not _rdb:
            _rdb = rdb.Rdb()

    return _rdb


class CeleryRdbGetPort(object):

    NAME = 'celery_bind'

    def __init__(self, arguments, factory):
        self.path = ArithmeticPath(arguments['path'])

        try:
            self._kwargs = arguments['rdb']
        except KeyError:
            self._kwargs = {}

    def execute(self, namespace):
        try:
            my_rdb = _get_rdb(**self._kwargs)
            self.path.write_to(namespace, PythonObjectNamespace(my_rdb.port))
        except Exception as exc:
            logger.exception("Failed to execute celery Rdb")


class CeleryRdb(object):

    NAME = 'celery_rdb'

    def __init__(self, arguments, factory):
        try:
            self._kwargs = arguments['rdb']
        except KeyError:
            self._kwargs = {}

    def execute(self, namespace):
        try:
            my_rdb = _get_rdb(**self._kwargs)
            my_rdb.connect()
            my_rdb.set_trace()
        except Exception as exc:
            logger.exception("Failed to execute celery Rdb")



import six
import traceback
import sys
import datetime
import time
import threading
import os
import inspect

from ..processor.namespaces.python_object_namespace import PythonObjectNamespace

from rook.config import LoggingConfiguration

LOG_LEVELS = ["trace", "debug", "info", "warning", "error", "fatal"]

TRACEBACK_LIMIT = 25

STDERR_WRITABLE = True


class _LogRecord(object):

    def __init__(self, level, level_no, text, formatted_message, args):
        self.level = level
        self.level_no = level_no
        self.datetime = datetime.datetime.now()
        self.time = time.time()
        self.filename, self.lineno, self.function = self._get_caller()
        self.text = text
        self.formatted_message = formatted_message
        self.args = args
        self.process = os.getpid()
        self.thread = threading.current_thread().ident
        self.thread_name = threading.current_thread().name

    def format(self):
        return r'%s %d:%s- %s:%s@%d - %s - %s' % (
            str(self.datetime),
            self.process,
            self.thread_name,
            os.path.splitext(self.filename)[0],
            self.function,
            self.lineno,
            self.level,
            self.formatted_message)

    # Based off Python built-in logging library

    # _srcfile is used when walking the stack to check when we've got the first
    # caller stack frame.
    #
    _srcfile = os.path.normcase(sys._getframe(1).f_code.co_filename)

    @staticmethod
    def _get_caller():
        """
        Find the stack frame of the caller so that we can note the source
        file name, line number and function name.
        """
        f = inspect.currentframe()
        # On some versions of IronPython, currentframe() returns None if
        # IronPython isn't run with -X:Frames.
        if f is not None:
            f = f.f_back
        rv = "(unknown file)", 0, "(unknown function)"
        while hasattr(f, "f_code"):
            co = f.f_code
            filename = os.path.normcase(co.co_filename)
            if filename == _LogRecord._srcfile:
                f = f.f_back
                continue
            rv = (co.co_filename, f.f_lineno, co.co_name)
            break
        return rv


class Logger(object):

    def __init__(self):
        if 'unittest' in sys.argv[0]:
            LoggingConfiguration.LOG_TO_STDERR = True
            LoggingConfiguration.LOG_LEVEL = "debug"

        self.output = None

        try:
            self.verbosity = LOG_LEVELS.index(LoggingConfiguration.LOG_LEVEL.lower())
        except ValueError:
            self.verbosity = LOG_LEVELS.index("info")

        self.handlers = self._build_handlers()

    def register_output(self, output):
        self.output = output

    def remove_output(self, output):
        self.output = None

    def debug(self, message, *args, **kwargs):
        self._log("debug", message, args, exc_info=kwargs.get('exc_info', False))

    def info(self, message, *args, **kwargs):
        self._log("info", message, args, exc_info=kwargs.get('exc_info', False))

    def warn(self, message, *args, **kwargs):
        self._log("warning", message, args, exc_info=kwargs.get('exc_info', False))

    def warning(self, message, *args, **kwargs):
        self._log("warning", message, args, exc_info=kwargs.get('exc_info', False))

    def error(self, message, *args, **kwargs):
        self._log("error", message, args, exc_info=kwargs.get('exc_info', False))

    def exception(self, message, *args):
        self._log("error", message, args, exc_info=1)

    def _log(self, level, message, args, **kwargs):
        try:
            level_no = len(LOG_LEVELS)
            try:
                level_no = LOG_LEVELS.index(level)
            except ValueError:
                pass
                # Use default value (error)

            if level_no < self.verbosity:
                return

            formatted_message = message
            try:
                formatted_message = message % args

                if kwargs.get("exc_info", False):
                    sys_exc_info = sys.exc_info()
                    formatted_message += traceback.format_exc(TRACEBACK_LIMIT)
            except Exception:
                pass
                # Use original message on error

            record = _LogRecord(level, level_no, message, formatted_message, args)
            for handler in self.handlers:
                handler(record)

        except Exception:
            six.print_("[Rookout] Unexpected error when writing to log:")
            traceback.print_exc()

    def _build_handlers(self):
        handlers = []

        if LoggingConfiguration.LOG_TO_STDERR:
            handlers.append(self._get_stderr_handler())

        file_handler = self._get_file_handler(LoggingConfiguration.FILE_NAME)
        if file_handler:
            handlers.append(file_handler)

        handlers.append(self._remote_handler)

        return handlers

    @staticmethod
    def _get_file_handler(file_name):
        if not file_name:
            return None

        # Calculate file path
        abs_file_path = None
        if os.path.isabs(file_name):
            abs_file_path = file_name
        else:
            if "darwin" in sys.platform:
                abs_file_path = os.path.join(os.getenv("HOME", "."), file_name)
            elif sys.platform == "win32":
                abs_file_path = os.path.join(os.getenv("USERPROFILE", "."), file_name)
            else:
                abs_file_path = os.path.join("/var/log", file_name)

        try:
            # Create directory if does not exist
            dirname = os.path.dirname(abs_file_path)
            if not os.path.isdir(dirname):
                os.makedirs(dirname)

            # file handler
            f = open(abs_file_path, "wt")
            return lambda record: f.write(record.format() + "\n")
        except Exception:
            if LoggingConfiguration.DEBUG:
                six.print_("[Rookout] Failed to open log file: %s".format(abs_file_path))
                traceback.print_exc()

    @staticmethod
    def _get_stderr_handler():
        if six.PY3:
            # In some versions of Python3, trying to write to STDERR in a
            # daemon thread after the main thread has ended might
            # lead to crashes.
            import atexit

            def disable_writing():
                global STDERR_WRITABLE
                STDERR_WRITABLE = False

            atexit.register(disable_writing)

        def _stderr_handler(record):
            if sys and sys.stderr and STDERR_WRITABLE:
                sys.stderr.write(record.format() + "\n")

        return _stderr_handler

    def _remote_handler(self, record):
        if not self.output:
            return

        try:
            arguments = {}

            if record.args:
                arguments['args'] = PythonObjectNamespace(record.args)

            self.output.send_log_message(
                record.level_no,
                record.time,
                record.filename,
                record.lineno,
                record.text,
                record.formatted_message,
                arguments)
        except Exception:
            pass


logger = Logger()



from six import iteritems

from ...logger import logger

from ..namespaces.python_object_namespace import PythonObjectNamespace
from ..namespaces.container_namespace import ContainerNamespace


class Eval(object):

    def __init__(self, arguments, factory):
        self._paths = list()

        for key, value in iteritems(arguments['paths']):
            try:
                dest_path = factory.get_path(key)
                source_path = value
            except Exception:
                logger.exception("Failed to load dest:source path pair %s:%s", key, value)
                continue

            self._paths.append((dest_path, source_path))

    def execute(self, namespace):
        if isinstance(namespace, ContainerNamespace):
            eval_locals = namespace
        else:
            eval_locals = {}

        for dest_path, source_path in self._paths:
            try:
                value = eval(source_path, eval_locals)
                if isinstance(value, PythonObjectNamespace):
                    value.dump_config = PythonObjectNamespace.ObjectDumpConfig.tailor_limits(value.obj)

                dest_path.write_to(namespace, value)
            except Exception:
                logger.exception("Failed to execute dest:source path pair")
                continue



"""This module is in charge of managing the rook's module state.

The external interface for the module is Rook located in rook.rook."""

import os
import sys
import platform
import atexit
import uuid

# This must be the first import, otherwise enable_gevent_for_grpc will fail since grpc has already been imported
from rook.logger import logger

from rook.config import AgentComConfiguration

from rook.com_ws.output_ws import Output

from .augs.augs_manager import AugsManager
from .trigger_services import TriggerServices

from rook.exceptions import RookInterfaceException, RookVersionNotSupported
from rook.com_ws.agent_com_ws import AgentCom
from rook.com_ws.command_handler import CommandHandler

try:
    from .atfork import install_fork_handler, remove_fork_handler
except ImportError:
    pass


class _Singleton(object):
    """This is singleton is the class managing the module.

    It should never be referred to directly, instead use obj in this module."""

    def __init__(self):
        """Initialize the object, sets member variables."""
        self._check_version_supported()

        logger.info("Initializing Rook under process-%d", os.getpid())

        self._id = None
        self._services_started = False

        self._trigger_services = TriggerServices()
        self._command_handler = None
        self._aug_manager = None
        self._fork_handler = None

        self._agent_com = None

    def _start_trigger_services(self):
        """Start trigger services.

        Calling this method multiple times will have no effect.
        """
        # Don't double init services
        if self._services_started:
            return

        self._init_id()
        self._output = Output(self._id)

        self._trigger_services.start()
        self._aug_manager = AugsManager(self._trigger_services, self._output)
        self._services_started = True

    def _init_id(self):
        self._id = uuid.uuid4().hex

    def _stop_trigger_services(self):
        if not self._services_started:
            return

        self._aug_manager = None
        self._trigger_services.close()

        self._services_started = False

    def get_trigger_services(self):
        return self._trigger_services

    def connect(self, token, host, port, proxy, tags=None, labels=None, async_start=False, fork=False):
        """Connect to the Agent."""
        if self._agent_com:
            raise RookInterfaceException("Multiple connection attempts not supported!")

        if install_fork_handler:
            install_fork_handler(fork)

        logger.debug("Initiating AgentCom-\t%s:%d", host, int(port))

        labels = labels or {}
        tags = tags or []

        self._start_trigger_services()
        self._agent_com = AgentCom(self._id, host, port, proxy, token, labels, tags)
        self._command_handler = CommandHandler(self._agent_com, self._aug_manager)
        self._output.set_agent_com(self._agent_com)
        self._agent_com.start()
        if async_start is False:
            self._agent_com.wait_for_ready(AgentComConfiguration.TIMEOUT)

    def flush(self):
        self._output.flush_messages()

    def stop(self):
        logger.debug("Shutting down")

        if self._agent_com is not None:
            self._agent_com.stop()
        self._agent_com = None

        self._stop_trigger_services()

    def restart(self, tags=None, labels=None):
        logger.debug("Restarting")
        if not self._agent_com:
            raise RookInterfaceException("Could not restart connection since it was not started")

        self.flush()

        self._init_id()
        self._output.set_agent_id(self._id)
        self._agent_com.update_info(self._id, tags, labels)

        self._agent_com.restart()

    def pre_fork(self):
        # AgentCom is fork safe (the child will lose the thread) so there's nothing we need to do with it
        self._trigger_services.pre_fork()

    def post_fork_recover(self):
        self._trigger_services.post_fork()

    def post_fork_clean(self):
        self._trigger_services.post_fork()

        self._command_handler = None

        if self._agent_com:
            self._agent_com.stop()
            self._agent_com = None

        self._stop_trigger_services()
        if remove_fork_handler:
            remove_fork_handler()

    @staticmethod
    def _check_version_supported():
        try:
            supported_platforms = ['pypy', 'cpython']
            supported_version = ['2.7', '3.5', '3.6', '3.7', '3.8']

            current_platform = platform.python_implementation().lower()
            if current_platform not in supported_platforms:
                raise RookVersionNotSupported("Rook is not supported in this platform: " + current_platform)

            major, minor, _, _, _ = sys.version_info
            current_version = "{}.{}".format(major, minor)
            if current_version not in supported_version:
                raise RookVersionNotSupported("Rook is not supported in this python version: " + current_version)

        except Exception as e:
            import traceback
            traceback.print_exc()

            raise e


singleton_obj = _Singleton()


def exit_handler():
    try:
        logger.info("Exit handler called - flushing and closing WebSocket")
        if singleton_obj._agent_com:
            singleton_obj.flush()
            singleton_obj._agent_com.stop()
    except:
        logger.exception("Flush and close failed")
    logger.info("Exit handler finished")


atexit.register(exit_handler)




from rook.processor.namespaces.python_object_namespace import PythonObjectNamespace
from rook.processor.namespaces.container_namespace import ContainerNamespace


class LogRecordNamespace(PythonObjectNamespace):

    def __init__(self, record):
        super(LogRecordNamespace, self).__init__(record, methods=self.METHODS)

    def format(self, args=None):
        return PythonObjectNamespace(self.obj.getMessage())

    def dump(self, args=None):
        return ContainerNamespace({
            'name': PythonObjectNamespace(self.obj.name),
            'msg': PythonObjectNamespace(self.obj.msg),
            'formatted_message': self.format(),
            'args': PythonObjectNamespace(self.obj.args),
            'level_name': PythonObjectNamespace(self.obj.levelname),
            'level_no': PythonObjectNamespace(self.obj.levelno),
            'path_name': PythonObjectNamespace(self.obj.pathname),
            'filename': PythonObjectNamespace(self.obj.filename),
            'module': PythonObjectNamespace(self.obj.module),
            'lineno': PythonObjectNamespace(self.obj.lineno),
            'function': PythonObjectNamespace(self.obj.funcName),
            'time': PythonObjectNamespace(self.obj.created),
            'thread_id': PythonObjectNamespace(self.obj.thread),
            'thread_name': PythonObjectNamespace(self.obj.threadName),
            'process_name': PythonObjectNamespace(self.obj.processName),
            'process_id': PythonObjectNamespace(self.obj.process)
        })

    METHODS = (format, dump)



"""
Loads Rook into pyspark workers
Usage: spark-submit --conf spark.python.daemon.module=rook.pyspark_daemon
"""
import types
import pyspark.daemon
import functools
import six
import sys
import os
import itertools

from rook.config import ImportServiceConfig

original_worker_main = pyspark.daemon.worker_main


def worker_main(*args, **kwargs):
    try:
        import rook
        rook.start(log_file="", log_to_stderr=True)
        from rook.logger import logger
        logger.debug("Started Rook in Spark worker")
        from rook.interface import _rook as singleton, _TRUE_VALUES
        from rook.services import ImportService
        import_service = singleton.get_trigger_services().get_service(ImportService.NAME)

        def pickle_load_hook(orig_func, *args, **kwargs):
            obj = orig_func(*args, **kwargs)
            try:
                # this is here to deal with the delay of having the periodic thread call evaluate_module_list -
                # it could miss a module being imported.
                import_service.evaluate_module_list()
            except:
                logger.exception("Silenced exception during module list evaluation")
            return obj

        # we may end up missing pickle module imports if we rely on the sys.modules polling thread
        import pyspark.serializers
        if ImportServiceConfig.USE_IMPORT_HOOK is False:  # only do this if we're not using the import hook
            pyspark.serializers.pickle.loads = functools.partial(pickle_load_hook, pyspark.serializers.pickle.loads)
            pyspark.serializers.pickle.load = functools.partial(pickle_load_hook, pyspark.serializers.pickle.load)
    except:
        six.print_("Starting Rook in worker_main failed", file=sys.stderr)

    result = original_worker_main(*args, **kwargs)
    return result


pyspark.daemon.worker_main = worker_main

if __name__ == '__main__':
    pyspark.daemon.manager()



from six import iteritems

from ...logger import logger

from ..namespaces.python_object_namespace import PythonObjectNamespace
from ..namespaces.container_namespace import ContainerNamespace


class Exec(object):

    def __init__(self, arguments, factory):
        self._statements = arguments['statements']

    def execute(self, namespace):
        if isinstance(namespace, ContainerNamespace):
            exec_locals = namespace
        else:
            exec_locals = {}

        try:
            exec(self._statements, exec_locals)
        except Exception:
            logger.exception("exec threw an exception")



import sys
import os
import logging
import platform
from rook.exceptions.tool_exceptions import RookDependencyError

PLATFORM = sys.platform
CPYTHON = platform.python_implementation() == 'CPython'


class DummyExtension(object):
    pass


if PLATFORM in ('darwin', 'linux2', 'linux'):
    hook_installed = False
    original_os_fork = os.fork
    _fork = None

    if CPYTHON:
        try:
            import native_extensions
        except Exception as e:
            raise RookDependencyError(e)
    else:
        native_extensions = DummyExtension()

    native_extensions.python_fork_handler_called = 1

    def os_fork_hook():
        global _fork
        try:
            from .singleton import singleton_obj

            singleton_obj.pre_fork()
        except:
            # TODO - remove
            import traceback
            traceback.print_exc()
            pass

        # Set "called" flag, to be checked in the
        # pthread_atfork, and reset after
        native_extensions.python_fork_handler_called = 1
        pid = original_os_fork()
        native_extensions.python_fork_handler_called = 0

        try:
            from .singleton import singleton_obj

            if pid == 0:
                # child

                # Clean all Rook state
                singleton_obj.post_fork_clean()

                # Shutdown external API
                from . import interface
                interface.stop()

                # restore original fork
                os.fork = original_os_fork

                # At this point the rook is fully down
                if _fork:
                    interface.start()
            else:
                # parent
                singleton_obj.post_fork_recover()
        except:
            # TODO - remove
            import traceback
            traceback.print_exc()
            pass

        return pid

    def install_fork_handler(fork):
        global hook_installed, original_os_fork, _fork

        if hook_installed:
            return

        if CPYTHON:
            # due to occasional deadlocks in PyPy, pthread_atfork
            # functionality is disabled
            native_extensions.RegisterPreforkCallback()

        os.fork, hook_installed, _fork = os_fork_hook, True, fork

    def remove_fork_handler():
        global hook_installed, original_os_fork

        os.fork, hook_installed = original_os_fork, False

else:
    def install_fork_handler(fork):
        pass



import inspect
import hashlib
import zlib

import six
import os

from rook.services.bdb_location_service import BdbLocationService
from rook.services.import_service import ImportService

from rook.exceptions import RookHashMismatchException, \
    RookCrcMismatchException, RookLineMoved

from rook.logger import logger
from rook.processor.error import Error
from rook.file_utils import FileUtils

class LocationFileLine(object):

    NAME = 'file_line'

    def __init__(self, arguments, processor_factory=None):
        self.filename = arguments.get('filename')
        # If we have a filename, make sure pyc is removed
        if self.filename:
            if isinstance(self.filename, six.string_types):
                # Some BDBs identify files by ID
                self.filename = self.filename.replace('.pyc', '.py')

        if self.filename is None:
            self.module_name = arguments['module_name'].lower()
        else:
            self.module_name = None

        self.lineno = arguments['lineno']

        self.file_hash = arguments.get('sha256')
        self.line_crc32 = arguments.get('line_crc32_2')
        self.line_unique = arguments.get('line_unique', False)

        self.file_utils = FileUtils()

        self.include_externals = arguments.get('includeExternals')
        if self.include_externals is None:
            # The only case it will be equal to '' is when its only filename (without folder, and without '/')
            if self.filename and isinstance(self.filename, six.string_types) and os.path.dirname(self.filename) == '':
                self.include_externals = False
            else:
                self.include_externals = True

    def add_aug(self, trigger_services, output, aug):
        logger.info("Adding aug")

        def callback(module):
            try:
                updatedLineNo = self._get_updated_line_number(aug, module)

                filepath = inspect.getsourcefile(module)
                if not filepath:
                    if module.__file__.endswith('.pyc'):
                        filepath = module.__file__.replace('.pyc', '.py')

                trigger_services.get_service(BdbLocationService.NAME).add_breakpoint_aug(module, updatedLineNo,
                                                                                         filepath, aug)
            except Exception as exc:
                message = "Exception when adding aug"
                logger.exception(message)
                aug.set_error(Error(exc=exc, message=message))

        def removed():
            aug.set_removed()

        aug.set_pending()
        trigger_services.get_service(ImportService.NAME).register_post_import_notification(
            aug, self.module_name, self.filename, self.file_hash, self.include_externals, self.lineno, callback, removed)

    def _get_updated_line_number(self, aug, module):
        filepath = inspect.getsourcefile(module)
        file_contents = self.file_utils.get_file_contents(aug, module, filepath)
        if file_contents is None:
            return self.lineno

        if self.line_crc32 is not None:
            return self._find_line_using_crc(aug, filepath, file_contents)

        if self.file_hash:
            self._verify_file_hash(filepath, file_contents)
            return self.lineno

        return self.lineno

    def _find_line_using_crc(self, aug, filepath, file_contents):
        lines = file_contents.splitlines()

        line_crc32 = None
        if len(lines) >= self.lineno:
            line_crc32 = format(zlib.crc32(lines[self.lineno - 1]) & 0xffffffff, 'x')
        if line_crc32 == self.line_crc32:
            return self.lineno

        if self.line_unique:
            crc32s = [format(zlib.crc32(line) & 0xffffffff, 'x') for line in lines]
            firstIndex = None
            secondFound = False

            for i in range(len(crc32s)):
                if crc32s[i] == self.line_crc32:
                    if firstIndex is None:
                        firstIndex = i
                    else:
                        secondFound = True
                        break

            if firstIndex and not secondFound:
                updatedLine = firstIndex + 1
                aug.send_warning(Error(exc=RookLineMoved(filepath, self.lineno, updatedLine)))
                return updatedLine

        blob_hash = self._get_git_blob_hash(filepath)
        raise RookCrcMismatchException(filepath, self.line_crc32, line_crc32, blob_hash)

    def _verify_file_hash(self, filepath, file_contents):
        hash = hashlib.sha256(file_contents).hexdigest()
        if hash != self.file_hash:
            blob_hash = self._get_git_blob_hash(filepath)
            raise RookHashMismatchException(filepath, self.file_hash, hash, blob_hash)

    def _get_git_blob_hash(self, file_path):
        return None



import six

from .namespace import Namespace
from .python_object_namespace import PythonObjectNamespace


class DumpedObjectNamespace(Namespace):

    def __init__(self, type, common_type, attributes, methods=()):
        super(DumpedObjectNamespace, self).__init__(methods + (DumpedObjectNamespace.type, ))
        self.type = type
        self.common_type = common_type
        self.attributes = attributes

    def read_attribute(self, name):
        return self.attributes[name]

    def no_attriubtes(self):
        return 0 == len(self.attributes)

    def read_key(self, key):
        raise NotImplementedError()

    def type(self, args=None):
        return PythonObjectNamespace(self.type)

    def serialize_type(self):
        return type



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: envelope.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='envelope.proto',
  package='com.rookout',
  syntax='proto3',
  serialized_options=b'\n\031com.rookout.rook.protobuf',
  serialized_pb=b'\n\x0e\x65nvelope.proto\x12\x0b\x63om.rookout\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x19google/protobuf/any.proto\"\\\n\x08\x45nvelope\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12!\n\x03msg\x18\x02 \x01(\x0b\x32\x14.google.protobuf.AnyB\x1b\n\x19\x63om.rookout.rook.protobufb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_protobuf_dot_any__pb2.DESCRIPTOR,])




_ENVELOPE = _descriptor.Descriptor(
  name='Envelope',
  full_name='com.rookout.Envelope',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='com.rookout.Envelope.timestamp', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='msg', full_name='com.rookout.Envelope.msg', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=91,
  serialized_end=183,
)

_ENVELOPE.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ENVELOPE.fields_by_name['msg'].message_type = google_dot_protobuf_dot_any__pb2._ANY
DESCRIPTOR.message_types_by_name['Envelope'] = _ENVELOPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Envelope = _reflection.GeneratedProtocolMessageType('Envelope', (_message.Message,), {
  'DESCRIPTOR' : _ENVELOPE,
  '__module__' : 'envelope_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.Envelope)
  })
_sym_db.RegisterMessage(Envelope)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import six

from .dumped_object_namespace import DumpedObjectNamespace


class UserObjectNamespace(DumpedObjectNamespace):

    def __init__(self, type, attributes):
        super(UserObjectNamespace, self).__init__(type, u'UserObject', attributes)



"""This module is in charge of building a Aug object from it's over the wire form.

Since most serialization format do not support polymorphism, we add this capability in this module.

This module finds and loads all dynamic classes and dynamically builds the relevant component based on it's id.
"""

import re

import six

from rook.factory import Factory

from ..augs import actions, conditions, extractors, locations
from ..augs.aug import Aug

from .processor_extensions.operations.celery_rdb import CeleryRdb, CeleryRdbGetPort


from rook.rookout_json import json
from rook import config

from rook.processor.processor_factory import ProcessorFactory

from rook.exceptions import ToolException, RookAugInvalidKey, RookObjectNameMissing, RookUnknownObject, RookInvalidObjectConfiguration, RookUnsupportedLocation


class AugFactory(Factory):
    """This is the factory for building Augs by their configuration."""

    def __init__(self, output):
        """Initialize the class."""
        super(AugFactory, self).__init__()

        self._output = output
        self._processor_factory = ProcessorFactory([CeleryRdb, CeleryRdbGetPort], [])

        self._load_dynamic_classes()

    @staticmethod
    def get_dict_value(configuration, value, default_value):
        val = configuration.get(value)
        return val if val is not None else default_value

    def get_aug(self, configuration):
        """Returns an Aug object based on the given configuration."""
        try:
            aug_id = configuration['id']
        except KeyError as exc:
            six.raise_from(RookAugInvalidKey('id', json.dumps(configuration)), exc)

        try:
            locationDict = configuration['location']
        except KeyError as exc:
            six.raise_from(RookAugInvalidKey('location', json.dumps(configuration)), exc)
        location = self._get_dynamic_class(locationDict)

        if 'extractor' in configuration:
            extractor = self._get_dynamic_class(configuration['extractor'])
        else:
            extractor = None

        condition = None
        conditional = configuration.get('conditional')
        if conditional:
            condition = conditions.IfCondition(conditional)

        try:
            actionDict = configuration['action']
        except KeyError as exc:
            six.raise_from(RookAugInvalidKey('action', json.dumps(configuration)), exc)
        action = self._get_dynamic_class(actionDict)

        limits_spec = configuration.get('rateLimit', None)
        rate_limits = None

        if limits_spec:
            r = re.match(r'^(\d+)/(\d+)$', limits_spec)

            if r is not None:
                rate_limits = (int(r.group(1)), int(r.group(2)))

        if not rate_limits:
            rate_limits = (200, 5000)

        max_aug_execution_time = AugFactory.get_dict_value(configuration, 'maxAugTime', config.InstrumentationConfig.MAX_AUG_TIME)

        return Aug(aug_id=aug_id,
                   location=location,
                   extractor=extractor,
                   condition=condition,
                   action=action,
                   output=self._output,
                   max_aug_execution_time=max_aug_execution_time,
                   rate_limits=rate_limits)

    def _load_dynamic_classes(self):
        """Load all dynamic classes into the factory."""
        self.register_methods(locations.__all__)
        self.register_methods(extractors.__all__)
        self.register_methods(conditions.__all__)
        self.register_methods(actions.__all__)

    def _get_dynamic_class(self, configuration):
        """Return a class based on configuration."""

        if not configuration:
            return None
        else:
            try:
                name = configuration['name']
            except KeyError as exc:
                six.raise_from(RookObjectNameMissing(json.dumps(configuration)), exc)

            try:
                factory = self.get_object_factory(name)
            except (RookUnknownObject, AttributeError, KeyError) as exc:
                six.raise_from(RookUnsupportedLocation(name), exc)

            try:
                return factory(configuration, self._processor_factory)
            except ToolException:
                raise
            except Exception as exc:
                six.raise_from(RookInvalidObjectConfiguration(name, json.dumps(configuration)), exc)



import socket
import threading
import time
from six.moves.queue import Queue, Empty
from six.moves.urllib.parse import urlparse
import select
import errno
import certifi
import ssl
import re
import os
import six
import websocket

from rook.exceptions import RookCommunicationException, RookInvalidToken, RookDependencyConflict
import rook.com_ws.socketpair_compat  # do not remove - adds socket.socketpair on Windows

try:
    from websocket import WebSocketBadStatusException
except ImportError:
    raise RookDependencyConflict('websocket')

# Python < 2.7.9 is missing important SSL features for websocket
# (unless supplied by CentOS etc)
if not websocket._ssl_compat.HAVE_SSL:
    try:
        import backports.ssl
        import backports.ssl_match_hostname
        import OpenSSL
        websocket._http.ssl = backports.ssl
        websocket._http.HAVE_SSL = True
        websocket._http.HAVE_CONTEXT_CHECK_HOSTNAME = True
    except ImportError:
        six.print_('[Rookout] Python is missing modern SSL features. To rectify, please run:\n'
                   '  pip install rook[ssl_backport]')

from rook.com_ws import information
from rook.logger import logger
import rook.protobuf.messages_pb2 as messages_pb
import rook.protobuf.envelope_pb2 as envelope_pb
from rook.config import AgentComConfiguration, VersionConfiguration


def wrap_in_envelope(message):
    envelope = envelope_pb.Envelope()
    envelope.timestamp.GetCurrentTime()
    envelope.msg.Pack(message)

    return envelope


class ExitThreadSentinel(object):
    pass


class FlushMessagesEvent(object):
    def __init__(self):
        self.event = threading.Event()


class MessageCallback(object):
    def __init__(self, cb, persistent):
        self.cb = cb
        self.persistent = persistent


class AgentCom(object):
    def __init__(self, agent_id, host, port, proxy, token, labels, tags):
        self.id = agent_id

        self._init_connect_thread()

        self._host = host if '://' in host else 'ws://' + host
        self._port = port
        self._proxy = proxy
        self._token = token
        self._token_valid = False
        self._labels = labels or {}
        self._tags = tags or []

        self._loop = None
        self._connection = None
        self._queue = Queue()

        self._running = True

        self._ready_event = threading.Event()
        self._connection_error = None

        self._callbacks = {}

        def set_ready_event(*args):
            self._ready_event.set()

        self.once('InitialAugsCommand', set_ready_event)
        self.poll_available = hasattr(select, "poll")

    def start(self):
        self._thread.start()

    def stop(self):
        self._running = False

        if self._connection is not None:
            self._connection.close(1000)

    def restart(self):
        self.stop()
        self._thread.join()

        self._connection = None

        self._init_connect_thread()

        self._running = True
        self.start()

    def update_info(self, agent_id, tags, labels):
        self.id = agent_id
        self._labels = labels or {}
        self._tags = tags or []

    def add(self, message):
        if self._queue.qsize() < AgentComConfiguration.MAX_QUEUED_MESSAGES:
            self._queue.put(wrap_in_envelope(message))

    def on(self, message_name, callback):
        self._register_callback(message_name, MessageCallback(callback, True))

    def once(self, message_name, callback):
        self._register_callback(message_name, MessageCallback(callback, False))

    def await_message(self, message_name):
        event = threading.Event()
        self.once(message_name, lambda _: event.set())

        return event

    def wait_for_ready(self, timeout=None):
        if not self._ready_event.wait(timeout):
            raise RookCommunicationException()
        else:
            if self._connection_error is not None:
                raise self._connection_error

    def _connect(self):
        retry = 0
        backoff = AgentComConfiguration.BACK_OFF
        connected = False
        last_successful_connection = 0

        while self._running:
            try:
                try:
                    if connected and time.time() >= last_successful_connection + AgentComConfiguration.RESET_BACKOFF_TIMEOUT:
                        retry = 0
                        backoff = AgentComConfiguration.BACK_OFF

                    self._connection = self._create_connection()

                    self._register_agent()

                except websocket.WebSocketBadStatusException as e:
                    if not self._token_valid and e.status_code == 403:  # invalid token
                        self._connection_error = RookInvalidToken(self._token)
                        self._ready_event.set()

                        logger.error('Connection failed; %s', self._connection_error.get_message())
                    raise
            except Exception as e:
                retry += 1
                backoff = min(backoff * 2, AgentComConfiguration.MAX_SLEEP)
                connected = False

                if hasattr(e, 'message') and e.message:
                    reason = e.message
                else:
                    reason = str(e)

                logger.info('Connection failed; reason = %s, retry = #%d, waiting %.3fs', reason, retry, backoff)

                time.sleep(backoff)
                continue
            else:
                connected = True
                last_successful_connection = time.time()

            logger.debug("WebSocket connected successfully")
            self._token_valid = True

            stop_socket_incoming, incoming_client_socket = socket.socketpair()
            outgoing_exit_sentinel = ExitThreadSentinel()

            def signal_stop_incoming_thread():
                # Logging is not safe here, interpreter might be shutting down
                try:
                    stop_socket_incoming.send(b'1')
                except socket.error as socket_error:
                    if socket_error.errno == errno.EPIPE:
                        return

                    logger.debug("Failed to signal stop", exc_info=1)
                except Exception:
                    logger.debug("Failed to signal stop", exc_info=1)
                finally:
                    stop_socket_incoming.close()

            def signal_stop_outgoing_thread():
                # Logging is not safe here, interpreter might be shutting down
                try:
                    incoming_client_socket.close()
                except:
                    pass

                try:
                    self._queue.put(outgoing_exit_sentinel)
                except Exception:
                    logger.debug("Failed to signal stop", exc_info=1)

            routines = [threading.Thread(name="rookout-incoming-thread", target=self._incoming, args=(incoming_client_socket, signal_stop_outgoing_thread)),
                        threading.Thread(name="rookout-outgoing-thread", target=self._outgoing, args=(outgoing_exit_sentinel, signal_stop_incoming_thread))]

            got_initial_augs = self.await_message('InitialAugsCommand')

            for routine in routines:
                routine.daemon = True
                routine.start()

            if got_initial_augs.wait(AgentComConfiguration.TIMEOUT):
                logger.info("Finished initialization")
            else:
                logger.warning("Timeout waiting for initial augs")

            for routine in routines:
                routine.join()

            logger.debug("Reconnecting")

    def wait_for_event_poll(self, stop_socket):
        poller = select.poll()

        exception_flags = select.POLLHUP | select.POLLERR
        read_only_flags = select.POLLIN | select.POLLPRI | exception_flags

        # Registering all input close and exception types
        poller.register(self._connection, read_only_flags)
        poller.register(stop_socket, read_only_flags)

        events = poller.poll(AgentComConfiguration.PING_TIMEOUT * 1000)

        for fd, flag in events:
            if fd == stop_socket.fileno():
                return True  # signaled to stop
            elif fd == self._connection.fileno() and flag \
                    and flag & exception_flags:  # Checking these flags explicitly (ignore input data)
                return True

        if len(events) == 0:
            logger.debug("Incoming thread - ping timeout")
            return True  # select timed out - ping timeout

        # Every thing is ok
        return False

    def wait_for_event_select(self, stop_socket):
        rlist, _, xlist = select.select([self._connection, stop_socket],
                                        [],
                                        [self._connection, stop_socket],
                                        AgentComConfiguration.PING_TIMEOUT)
        if stop_socket in rlist or stop_socket in xlist or self._connection in xlist:
            return True  # signaled to stop

        if len(rlist) == 0 and len(xlist) == 0:
            logger.debug("Incoming thread - ping timeout")
            return True  # select timed out - ping timeout
        return False

    def _incoming(self, stop_socket, on_exit):
        try:
            while True:
                try:
                    # Poll has no file_descriptor limit (on linux) if available - use it
                    if self.poll_available:
                        if self.wait_for_event_poll(stop_socket):
                            break
                    else:
                        if self.wait_for_event_select(stop_socket):
                            break

                    # it wasn't stop_socket, so it's self._connection -> we can read. we have to read including
                    # control frames, otherwise select might return and recv() wouldn't actually return anything
                    code, msg = self._connection.recv_data(control_frame=True)
                    if code != websocket.ABNF.OPCODE_BINARY:
                        continue

                    if msg is None:
                        # socket disconnected
                        logger.debug("Incoming thread - socket disconnected")
                        break

                    envelope = envelope_pb.Envelope()
                    envelope.ParseFromString(msg)
                    self._handle_incoming_message(envelope)
                except (socket.error, websocket.WebSocketConnectionClosedException):
                    logger.debug("Incoming thread - socket disconnected")
                    break
        except:
            logger.exception("Incoming thread failed")

        finally:
            on_exit()

    def flush_all_messages(self):
        flush_event = FlushMessagesEvent()
        self._queue.put(flush_event)
        flush_event.event.wait(AgentComConfiguration.FLUSH_TIMEOUT)

    def _outgoing(self, outgoing_exit_sentinel, on_exit):
        try:
            last_ping = time.time()
            self._connection.ping()

            while True:
                msg = None
                if (time.time() - last_ping) >= AgentComConfiguration.PING_INTERVAL:
                    last_ping = time.time()
                    self._connection.ping()

                try:
                    msg = self._queue.get(timeout=AgentComConfiguration.PING_INTERVAL)
                    if isinstance(msg, ExitThreadSentinel):
                        if msg is outgoing_exit_sentinel:
                            break
                        continue  # if it's an ExitThreadSentinel but not from this specific thread, just skip it
                    if isinstance(msg, FlushMessagesEvent):
                        msg.event.set()
                        continue
                    self._send(msg)
                except Empty:
                    continue
                except (socket.error, websocket.WebSocketConnectionClosedException):
                    if msg is not None:
                        self._queue.put(msg)
                    break
        except:
            logger.exception("Outgoing thread failed")

        finally:
            on_exit()

    def _create_connection(self):
        url = '{}:{}/v1'.format(self._host, self._port)
        headers = {
            'User-Agent': 'RookoutAgent/{}+{}'.format(VersionConfiguration.VERSION, VersionConfiguration.COMMIT)
        }

        if self._token is not None:
            headers["X-Rookout-Token"] = self._token

        proxy_host, proxy_port = self._get_proxy()

        connect_args = (url,)
        connect_kwargs = dict(header=headers,
                              timeout=AgentComConfiguration.TIMEOUT,
                              http_proxy_host=proxy_host, http_proxy_port=proxy_port)

        if os.environ.get('ROOKOUT_NO_HOST_HEADER_PORT') == '1':
            host = re.sub(':\d+$', '', urlparse(url).netloc)
        else:
            host = None

        try:
            # connect using system certificates
            conn = websocket.create_connection(*connect_args, host=host, **connect_kwargs)
        # In some very specific scenario, you cannot
        # reference ssl.CertificateError because it does
        # exist, so instead we get with with getattr
        # (None never matches an exception)
        except (ssl.SSLError, getattr(ssl, 'CertificateError', None)):
            # connect using certifi certificate bundle
            # (Python 2.7.15+ from python.org on macOS rejects our CA, see RK-3383)
            connect_kwargs['sslopt'] = dict(ca_certs=certifi.where())
            logger.debug("Got SSL error when connecting using system CA cert store, falling back to certifi")
            conn = websocket.create_connection(*connect_args, **connect_kwargs)
        conn.settimeout(None)
        return conn

    def _get_proxy(self):
        if self._proxy is None:
            return None, None

        try:
            if not self._proxy.startswith("http://"):
                self._proxy = "http://" + self._proxy

            url = urlparse(self._proxy, "http://")

            logger.debug("Connecting via proxy: %s", url.netloc)

            return url.hostname, url.port
        except ValueError:
            return None, None

    def _register_agent(self):
        logger.info('Registering agent with id %s', self.id)
        info = information.collect()
        info.agent_id = self.id
        info.labels = self._labels
        info.tags = self._tags

        m = messages_pb.NewAgentMessage()
        m.agent_info.CopyFrom(information.pack_agent_info(info))

        return self._send(wrap_in_envelope(m))

    def _init_connect_thread(self):
        self._thread = threading.Thread(name="rookout-" + type(self).__name__, target=self._connect)
        self._thread.daemon = True

    AcceptedMessageTypes = [
        messages_pb.InitialAugsCommand,
        messages_pb.AddAugCommand,
        messages_pb.ClearAugsCommand,
        messages_pb.PingMessage,
        messages_pb.RemoveAugCommand
    ]

    def _handle_incoming_message(self, envelope):
        for message_type in self.AcceptedMessageTypes:
            if envelope.msg.Is(message_type.DESCRIPTOR):
                message = message_type()
                envelope.msg.Unpack(message)
                type_name = message.DESCRIPTOR.name

                callbacks = self._callbacks.get(type_name)

                if callbacks:
                    persistent_callbacks = []

                    # Trigger all persistent callbacks first
                    for callback in callbacks:
                        try:
                            if callback.persistent:
                                callback.cb(message)
                        except:
                            pass
                        finally:
                            if callback.persistent:
                                persistent_callbacks.append(callback)

                    # Trigger all non persistent callbacks
                    for callback in callbacks:
                        try:
                            if not callback.persistent:
                                callback.cb(message)
                        except:
                            pass

                    self._callbacks[type_name] = persistent_callbacks

    def _send(self, message):
        return self._connection.send_binary(message.SerializeToString())

    def _register_callback(self, message_name, callback):
        self._callbacks.setdefault(message_name, []).append(callback)



import six

from . import namespace


class DelayedNamespace(namespace.Namespace):
    def __init__(self, value):
        super(DelayedNamespace, self).__init__()

        self.value = value
        self.variant = None
        self.namespace = None

    def call_method(self, name, args):
        self._load()
        return self.namespace.call_method(name, args)

    def read_attribute(self, name):
        self._load()
        return self.namespace.read_attribute(name)

    def write_attribute(self, name, value):
        self._load()
        return self.namespace.write_attribute(name, value)

    def read_key(self, key):
        self._load()
        return self.namespace.read_key(key)

    def get_variant(self):
        self._load()
        return self.variant

    def _load(self):
        if self.namespace is not None:
            return

        from .. import namespace_serializer
        from google.protobuf.json_format import ParseDict, Parse
        from rook.protobuf import rook_pb2

        if isinstance(self.value, six.string_types):
            self.variant = Parse(self.value, rook_pb2.Variant())
        else:
            self.variant = ParseDict(self.value, rook_pb2.Variant())

        self.namespace = namespace_serializer.NamespaceSerializer().loads(self.variant)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: controller_info.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='controller_info.proto',
  package='com.rookout',
  syntax='proto3',
  serialized_options=b'\n\031com.rookout.rook.protobuf',
  serialized_pb=b'\n\x15\x63ontroller_info.proto\x12\x0b\x63om.rookout\"\xf6\x01\n\x15\x43ontrollerInformation\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\t\x12\x0c\n\x04tags\x18\x04 \x03(\t\x12\x10\n\x08hostname\x18\x05 \x01(\t\x12\x14\n\x0cmachine_type\x18\x06 \x01(\t\x12\x0f\n\x07network\x18\x07 \x01(\t\x12\n\n\x02ip\x18\x08 \x01(\t\x12\x10\n\x08platform\x18\t \x01(\t\x12\x18\n\x10platform_release\x18\n \x01(\t\x12\x18\n\x10platform_version\x18\x0b \x01(\t\x12\x17\n\x0fplatform_string\x18\x0c \x01(\tB\x1b\n\x19\x63om.rookout.rook.protobufb\x06proto3'
)




_CONTROLLERINFORMATION = _descriptor.Descriptor(
  name='ControllerInformation',
  full_name='com.rookout.ControllerInformation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='com.rookout.ControllerInformation.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='commit', full_name='com.rookout.ControllerInformation.commit', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='com.rookout.ControllerInformation.version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tags', full_name='com.rookout.ControllerInformation.tags', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hostname', full_name='com.rookout.ControllerInformation.hostname', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='machine_type', full_name='com.rookout.ControllerInformation.machine_type', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='network', full_name='com.rookout.ControllerInformation.network', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ip', full_name='com.rookout.ControllerInformation.ip', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='platform', full_name='com.rookout.ControllerInformation.platform', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='platform_release', full_name='com.rookout.ControllerInformation.platform_release', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='platform_version', full_name='com.rookout.ControllerInformation.platform_version', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='platform_string', full_name='com.rookout.ControllerInformation.platform_string', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=39,
  serialized_end=285,
)

DESCRIPTOR.message_types_by_name['ControllerInformation'] = _CONTROLLERINFORMATION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ControllerInformation = _reflection.GeneratedProtocolMessageType('ControllerInformation', (_message.Message,), {
  'DESCRIPTOR' : _CONTROLLERINFORMATION,
  '__module__' : 'controller_info_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.ControllerInformation)
  })
_sym_db.RegisterMessage(ControllerInformation)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
